#! env python

import math, sys
from common import *

def generate_geometry(view, i):
    map = ImageMap("texture.ppm")
    material = Diffuse(0.3, 0.5, 0.2)
    material.Kd = map
    n = 15
    for x in range(-n, n):
        for y in range(-n,n):
            for z in range(0,n):
                cube = Cube()
                s = math.fmod(x*x + y*y + z*z, 2)
                cube.l2w = matrix.Matrix()
                cube.l2w.set_scale(s, s, s)
                rx = (x + y) * (y + z) * 726
                ry = z * x + y*10 + z * 129
                rz = x * y * z + y * 39
                cube.l2w.rotate(rx, ry, rz)
                cube.l2w.translate(x*3 + math.fmod(z*321 + x*y,4)-2,
                                   y*3 + math.fmod(x*z+y*111,4)-2,
                                   z*3 + math.fmod(x*y*z + z*30,4)-2 - i/3.0)
                cube.l2w.rotate(8,12,0)
                cube.l2w_motion = cube.l2w.copy()
                cube.l2w.rotate(0, 0, i)
                cube.l2w_motion.translate(0, 0, 1.0/3.0)
                cube.l2w_motion.rotate(0, 0, i+1)
                view.add_geometry(cube, material, 3)

def frame(i):
    view = ZBufferView()
    view.set_format("1.85")
    view.scale_viewport(3)
    view.add_light(Direct(C=(1,1,1)))
    view.add_light(Direct(C=(1,1,1),  L=(0,0,-1)))
    view.set_samples(2)
    view.set_filter("gaussian", 2)
    render = view.get_image()
    imv = tui.ImageViewer(render)
    generate_geometry(view, i)
    view.render()

    # Composite render over a gradient background with dropshadowed text
    ip.match_viewport(render)
    foreground = ip.image(render)
    background = ip.gradient((0.7, 0.7, 1.0, 1, 1), (0.7, 0.7, 1.0, 1, 1),
                             (0.5, 0.5, 0.7, 1, 1), (0.5, 0.5, 0.7, 1, 1))
    comp = ip.matte(foreground, background).get_image()
#
# Only uncomment these lines if you have FreeType support enabled
#
#    ft = ip.Font("/usr/share/fonts/default/Type1/n019003l.pfb")
#    ft.set_size(16)
#    text = 'DisneyBorg                                                 %i' % i
#    ft.draw_string(comp, 19, 11, text, 0, 0, 0)
#    ft.draw_string(comp, 19, 9, text, 0, 0, 0)
#    ft.draw_string(comp, 21, 11, text, 0, 0, 0)
#    ft.draw_string(comp, 21, 9, text, 0, 0, 0)
#    ft.draw_string(comp, 20, 10, text, 1, 1, 1)
    text = 'cubegrid/m.%03d.ppm' % i
    comp.write(text)

def animate(min,max):
    print 'from ', min, 'to ', max
    for i in range(min, max):
        frame(i)

# This script is set up to run a single frame automatically:
animate(0, 1)

#
# Normally, I run this script while testing, so I comment out the
# "animate(0, 1)" statement above, start python, and then:
#
# >>> import animate
# >>> animate.animate(0, 1)
#
# Alternatively, you can uncomment the line below and run this
# script from the command line, with two arguments for the start
# and end frame numbers.
#
#animate(long(sys.argv[1]), long(sys.argv[2]))
