from parc import matrix, image, ip, tui
from parc import Geometry, Vertex, Material, Light, Scene
from random import uniform

class Cube(Geometry):
    def generate(self):
        v0 = Vertex((-0.5, -0.5, -0.5), (0,1))
        v1 = Vertex(( 0.5, -0.5, -0.5), (0,0))
        v2 = Vertex(( 0.5,  0.5, -0.5), (1,0))
        v3 = Vertex((-0.5,  0.5, -0.5), (1,1))
        self.quad(v1, v0, v3, v2)
        
        v0 = Vertex((-0.5, -0.5,  0.5), (0,0))
        v1 = Vertex(( 0.5, -0.5,  0.5), (0,1))
        v2 = Vertex(( 0.5,  0.5,  0.5), (1,1))
        v3 = Vertex((-0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex((-0.5,  0.5, -0.5), (0,0))
        v1 = Vertex((-0.5, -0.5, -0.5), (0,1))
        v2 = Vertex((-0.5, -0.5,  0.5), (1,1))
        v3 = Vertex((-0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex(( 0.5, -0.5, -0.5), (0,0))
        v1 = Vertex(( 0.5,  0.5, -0.5), (0,1))
        v2 = Vertex(( 0.5,  0.5,  0.5), (1,1))
        v3 = Vertex(( 0.5, -0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex(( 0.5,  0.5, -0.5), (0,0))
        v1 = Vertex((-0.5,  0.5, -0.5), (0,1))
        v2 = Vertex((-0.5,  0.5,  0.5), (1,1))
        v3 = Vertex(( 0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex((-0.5, -0.5, -0.5), (0,0))
        v1 = Vertex(( 0.5, -0.5, -0.5), (0,1))
        v2 = Vertex(( 0.5, -0.5,  0.5), (1,0))
        v3 = Vertex((-0.5, -0.5,  0.5), (1,1))
        self.quad(v0, v1, v2, v3)
        
        return 0

class Plane(Geometry):
    def generate(self):
        v0 = Vertex((-1, 0, -1), (0,1))
        v1 = Vertex(( 1, 0, -1), (0,0))
        v2 = Vertex(( 1, 0,  1), (1,0))
        v3 = Vertex((-1, 0,  1), (1,1))
        self.quad(v1, v0, v3, v2)
        return 0
    
class Diffuse(Material):
    def __init__(self, r, g, b, a=1):
        Material.__init__(self)
        self.Kd = (r, g, b)
        self.opacity = a
        
    def shade(self, shader):
        c = 0
        for light in shader.light_list:
            a = shader.N.dot(-light.L)
            if a > 0:
                c = c + a
        if c < 0.1: c = 0.3
        kd = shader.Kd
        return (c * kd.r, c * kd.g, c * kd.b, self.opacity)

class LocalDisplacement(Material):
    def __init__(self, scale=1):
        Material.__init__(self)
        self.scale = scale

    def shade(self, shader):
        p = shader.P + self.scale * shader.D.r * shader.N
        return (p.x, p.y, p.z, 1)
    
class ImageMap(Material):
    def __init__(self, map_name):
        Material.__init__(self)
        self.image = image.Image()
        d = self.image.read(map_name)
        if d < 0:
            print 'Cannot read texture map ', map_name, d
        else:
            print 'Read texture map ', map_name, d
        
    def shade(self, shader):
        c = self.image.get_color(shader.u, shader.v)
        return (c[0], c[1], c[2], 1)

class Ambient:
    def __init__(self, C=(1,1,1)):
        self.C = C

    def illuminate(self, shader):
        return self.C

class Direct:
    def __init__(self, C=(1,1,1), L=(0,-1,0)):
        self.L = L
        self.C = C

    def illuminate(self, shader):
        shader.L = self.L
        return self.C
    
def run(generate_geometry, name):
    scene = Scene()
    scene.set_format("1.85")
    scene.scale_viewport(3)
    scene.add_light(Direct(C=(1,1,1)))
    scene.add_light(Direct(C=(1,1,1),  L=(0,0,-1)))
    scene.set_samples(3)
    scene.set_filter("gaussian", 2)
#    scene.interpolate_mp(1)
    render = scene.get_image()
    imv = tui.ImageViewer(render)
    generate_geometry(scene)
    scene.render()

    # Composite render over a gradient background with dropshadowed text
    ip.match_viewport(render)
    foreground = ip.image(render)
    background = ip.gradient((0.5, 0.5, 0.9, 1, 1), (0.5, 0.5, 0.9, 1, 1),
                             (0.2, 0.2, 0.6, 1, 1), (0.2, 0.2, 0.6, 1, 1))
    comp = ip.matte(foreground, background).get_image()
# Unix    
    ft = ip.Font("/usr/share/fonts/default/Type1/n019003l.pfb")
# Linux
#    ft = ip.Font("C:\WINNT\Fonts\arial.ttf")
    ft.set_size(16)
    ft.draw_string(comp, 20, 10, name, 0.3, 0.3, 0.3)
    ft.draw_string(comp, 18, 12, name, 0.3,0.4,0.8)
    comp.write(name + ".ppm")

