from parc import image, Geometry, Vertex, Material, Light


class Cube(Geometry):
    def generate(self):
        v0 = Vertex((-0.5, -0.5, -0.5), (0,1))
        v1 = Vertex(( 0.5, -0.5, -0.5), (0,0))
        v2 = Vertex(( 0.5,  0.5, -0.5), (1,0))
        v3 = Vertex((-0.5,  0.5, -0.5), (1,1))
        self.quad(v1, v0, v3, v2)
        
        v0 = Vertex((-0.5, -0.5,  0.5), (0,0))
        v1 = Vertex(( 0.5, -0.5,  0.5), (0,1))
        v2 = Vertex(( 0.5,  0.5,  0.5), (1,1))
        v3 = Vertex((-0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex((-0.5,  0.5, -0.5), (0,0))
        v1 = Vertex((-0.5, -0.5, -0.5), (0,1))
        v2 = Vertex((-0.5, -0.5,  0.5), (1,1))
        v3 = Vertex((-0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex(( 0.5, -0.5, -0.5), (0,0))
        v1 = Vertex(( 0.5,  0.5, -0.5), (0,1))
        v2 = Vertex(( 0.5,  0.5,  0.5), (1,1))
        v3 = Vertex(( 0.5, -0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex(( 0.5,  0.5, -0.5), (0,0))
        v1 = Vertex((-0.5,  0.5, -0.5), (0,1))
        v2 = Vertex((-0.5,  0.5,  0.5), (1,1))
        v3 = Vertex(( 0.5,  0.5,  0.5), (1,0))
        self.quad(v0, v1, v2, v3)
        
        v0 = Vertex((-0.5, -0.5, -0.5), (0,0))
        v1 = Vertex(( 0.5, -0.5, -0.5), (0,1))
        v2 = Vertex(( 0.5, -0.5,  0.5), (1,0))
        v3 = Vertex((-0.5, -0.5,  0.5), (1,1))
        self.quad(v0, v1, v2, v3)
        
        return 0

class Plane(Geometry):
    def generate(self):
        v0 = Vertex((-1, 0, -1), (0,1))
        v1 = Vertex(( 1, 0, -1), (0,0))
        v2 = Vertex(( 1, 0,  1), (1,0))
        v3 = Vertex((-1, 0,  1), (1,1))
        self.quad(v1, v0, v3, v2)
        return 0
    
class Diffuse(Material):
    def __init__(self, r, g, b, a=1):
        Material.__init__(self)
        self.Kd = (r, g, b)
        self.opacity = a
        
    def shade(self, shader):
        c = 0
        for light in shader.light_list:
            a = shader.N.dot(-light.L)
            if a > 0:
                c = c + a
        if c < 0.1: c = 0.3
        kd = shader.Kd
        return (c * kd.r, c * kd.g, c * kd.b, self.opacity)

class LocalDisplacement(Material):
    def __init__(self, scale=1):
        Material.__init__(self)
        self.scale = scale

    def shade(self, shader):
        p = shader.P + self.scale * shader.D.r * shader.N
        return (p.x, p.y, p.z, 1)
    
class ImageMap(Material):
    def __init__(self, map_name):
        Material.__init__(self)
        self.image = image.Image()
        d = self.image.read(map_name)
        if d < 0:
            print 'Cannot read texture map ', map_name, d
        else:
            print 'Read texture map ', map_name, d
        
    def shade(self, shader):
        c = self.image.get_color(shader.u, shader.v)
        return (c[0], c[1], c[2], 1)

class Ambient:
    def __init__(self, C=(1,1,1)):
        self.C = C

    def illuminate(self, shader):
        return self.C

class Direct:
    def __init__(self, C=(1,1,1), L=(0,-1,0)):
        self.L = L
        self.C = C

    def illuminate(self, shader):
        shader.L = self.L
        return self.C
    
