from parc.matrix import Vector, Matrix
from parc import Geometry, Vertex, Material, Light, Scene
from random import *
from shaders import *

class velocity_curve:
    def __init__(self, p, v):
        self.position = Vector(p[0], p[1], p[2])
        self.velocity = Vector(v[0], v[1], v[2])

    def P(self, t):
        return self.position + t * self.velocity

def block_matrix(curve, scale, t):
    m = Matrix()
    m.scale(scale, scale, scale)
    p = curve.P(t)
    m.translate(p.x, p.y, p.z)
    return m

def generate_block(scene, curve, material, scale, t):
    cube = Cube()
    cube.l2w = block_matrix(curve, scale, t)
    cube.l2w_motion = block_matrix(curve, scale, t + 1)
    scene.add_geometry(cube, material, 3)

def cube_matrix(scale, p_curve, r_curve, t):
    m = Matrix()
    m.scale(scale, scale, scale)
    r = r_curve.P(t)
    m.rotate(r.x, r.y, r.z)
    p = p_curve.P(t)
    m.translate(p.x, p.y, p.z)
    return m

def generate_cubes(scene):
    if scene.frame < 5: return
    age = scene.frame - 5
    gravity = -0.0002 * age * age
    n = 20
    gg = Random(42)
    for i in range(n):
        r = gg.uniform(0.2, 1)
        g = gg.uniform(0.2, 1)
        b = gg.uniform(0.2, 1)
        material = Diffuse(r, g, b)
        scale = gg.uniform(0.1, 0.4)
        vx = gg.uniform(-0.05, 0.05)
        vy = gg.uniform(-0.05, 0.15) + gravity
        vz = gg.uniform(-0.05, 0.05)
        vry = gg.uniform(-2, 2)
        vrz = gg.uniform(-10, 10)
        p_curve = velocity_curve((0,0,0), (vx, vy, vz))
        r_curve = velocity_curve((0,0,0), (0, vry, vrz))
        cube = Cube()
        cube.l2w = cube_matrix(scale, p_curve, r_curve, scene.frame - 5)
        cube.l2w_motion = cube_matrix(scale, p_curve, r_curve, scene.frame - 4)
        scene.add_geometry(cube, material, 3)

def generate_bits(scene):
    if scene.frame < 5: return
    age = scene.frame - 5
    gravity = -0.0002 * age * age
    n = 500
    gg = Random(42)
    for i in range(n):
        material = Diffuse(0.5, 0.2, 0.2)
        scale = 0.05
        vx = gg.uniform(-0.3, 0.3)
        vy = gg.uniform(-0.3, 0.3) + gravity
        vz = gg.uniform(-0.3, 0.3)
        generate_block(scene, velocity_curve((0,0,0), (vx, vy, vz)),
                       material, scale, age)

def generate_geometry(scene):
    if scene.frame < 5:
        generate_block(scene,
                       velocity_curve((-4, 0, 0), (0.7, 0, 0)),
                       Diffuse(0.8, 0.4, 0.2), 0.75, scene.frame)
        generate_block(scene,
                       velocity_curve((4, 0, 0), (-0.7, 0, 0)),
                       Diffuse(0.4, 0.8, 0.2), 0.75, scene.frame)
    generate_cubes(scene)
    generate_bits(scene)

def generate_lights(scene):
    scene.add_light(Direct(L=(1,2,1)))


def generate(scene):
    generate_lights(scene)
    generate_geometry(scene)
